<?php
/**
 * Plugin main class.
 *
 * @package WPDesk\FlexibleWishlistAnalytics
 */

namespace WPDesk\FlexibleWishlistAnalytics;

use FlexibleWishlistAnalyticsVendor\WPDesk\PluginBuilder\Plugin\AbstractPlugin;
use FlexibleWishlistAnalyticsVendor\WPDesk\PluginBuilder\Plugin\HookableCollection;
use FlexibleWishlistAnalyticsVendor\WPDesk\PluginBuilder\Plugin\HookableParent;
use FlexibleWishlistAnalyticsVendor\WPDesk_Plugin_Info;
use WPDesk\FlexibleWishlist\Repository\SettingsRepository;
use WPDesk\FlexibleWishlist\Repository\WishlistRepository;
use WPDesk\FlexibleWishlistAnalytics\Endpoint\EmailCalculationEndpoint;
use WPDesk\FlexibleWishlistAnalytics\Endpoint\EndpointIntegrator;
use WPDesk\FlexibleWishlistAnalytics\Promotion\EmailMetaboxGenerator;
use WPDesk\FlexibleWishlistAnalytics\Promotion\FilterMetaboxGenerator;
use WPDesk\FlexibleWishlistAnalytics\Promotion\PostGenerator;
use WPDesk\FlexibleWishlistAnalytics\Promotion\PostTypeRegister;
use WPDesk\FlexibleWishlistAnalytics\Promotion\SenderMetaboxGenerator;
use WPDesk\FlexibleWishlistAnalytics\Service\WishlistFetcher;
use WPDesk\FlexibleWishlistAnalytics\Table\PopularCategoriesTable;
use WPDesk\FlexibleWishlistAnalytics\Table\PopularProductsTable;
use WPDesk\FlexibleWishlistAnalytics\Table\PopularTagsTable;
use WPDesk\FlexibleWishlistAnalytics\Table\TableLoader;
use WPDesk\FlexibleWishlistAnalytics\Table\WishlistsArchiveTable;

/**
 * @package WPDesk\FlexibleWishlistAnalytics
 */
class Plugin extends AbstractPlugin implements HookableCollection {
	use HookableParent;

	/**
	 * @var Service\TemplateLoader
	 */
	private $template_loader;

	public function __construct( WPDesk_Plugin_Info $plugin_info ) {
		parent::__construct( $plugin_info );
		$this->plugin_url       = $this->plugin_info->get_plugin_url();
		$this->plugin_namespace = $this->plugin_info->get_text_domain();
		$this->template_loader  = new Service\TemplateLoader( $plugin_info->get_plugin_dir(), 'flexible-wishlist' );
	}

	/**
	 * Integrate with WordPress and with other plugins using action/filter system.
	 */
	public function hooks(): void {
		parent::hooks();

		add_action(
			'flexible_wishlist/init',
			function ( SettingsRepository $settings_repository, WishlistRepository $wishlist_repository ) {
				$wishlist_fetcher = new WishlistFetcher( $wishlist_repository );
				$this->add_hookable( new EmailMetaboxGenerator() );
				$this->add_hookable( new FilterMetaboxGenerator() );
				$this->add_hookable( new SenderMetaboxGenerator( $wishlist_fetcher ) );
				$this->add_hookable( new PostTypeRegister() );
				$this->add_hookable( new PostGenerator() );
				$this->add_hookable( new TableLoader( $this->template_loader, new PopularProductsTable( $wishlist_fetcher ) ) );
				$this->add_hookable( new TableLoader( $this->template_loader, new PopularCategoriesTable( $wishlist_fetcher ) ) );
				$this->add_hookable( new TableLoader( $this->template_loader, new PopularTagsTable( $wishlist_fetcher ) ) );
				$this->add_hookable( new TableLoader( $this->template_loader, new WishlistsArchiveTable( $wishlist_fetcher, $wishlist_repository, $settings_repository ) ) );
				$this->add_hookable( new Settings\SettingsPageGenerator( $this->plugin_info ) );
				$this->add_hookable( new EndpointIntegrator( new EmailCalculationEndpoint( $wishlist_fetcher ) ) );

				$this->add_hookable( new Marketing\SupportPage( $this->template_loader ) );

				$this->hooks_on_hookable_objects();
			},
			10,
			2
		);
	}

	/**
	 * {@inheritdoc}
	 */
	public function links_filter( $links ): array {
		$plugin_links = [
			'<a href="' . esc_url( menu_page_url( 'flexible-wishlist-marketing', false ) ) . '" style="color:#007050;font-weight: bold">' . __( 'Start Here', 'flexible-wishlist-analytics' ) . '</a>',
			'<a href="' . admin_url( 'admin.php?page=flexible-wishlist-settings' ) . '">' . __( 'Settings', 'flexible-wishlist-analytics' ) . '</a>',
			'<a href="' . esc_url( __( 'https://wpde.sk/fw-settings-row-action-docs-pro', 'flexible-wishlist-analytics' ) ) . '" target="_blank">' . __( 'Docs', 'flexible-wishlist-analytics' ) . '</a>',
			'<a href="' . esc_url( __( 'https://wpde.sk/fw-settings-row-action-support-pro', 'flexible-wishlist-analytics' ) ) . '" target="_blank">' . __( 'Support', 'flexible-wishlist-analytics' ) . '</a>',
		];

		return array_merge( $plugin_links, $links );
	}
}
