<?php

namespace WPDesk\FlexibleWishlistAnalytics\Promotion;

use FlexibleWishlistAnalyticsVendor\WPDesk\PluginBuilder\Plugin\Hookable;
use WPDesk\FlexibleWishlistAnalytics\Endpoint\EmailCalculationEndpoint;
use WPDesk\FlexibleWishlistAnalytics\Filter\FilterAggregator;
use WPDesk\FlexibleWishlistAnalytics\Service\EmailSender;
use WPDesk\FlexibleWishlistAnalytics\Service\WishlistFetcher;

/**
 * .
 */
class SenderMetaboxGenerator implements Hookable {

	const SEND_PROMOTION_BUTTON_NAME       = 'wishlist_send_promotion';
	const SEND_PROMOTION_BUTTON_VALUE      = 'yes';
	const SEND_PROMOTION_BUTTON_VALUE_TEST = 'test';
	const INPUT_NAME_USERS                 = 'wishlist_users';
	const INPUT_NAME_NOTE                  = 'wishlist_note';
	const POST_META_PROMOTION_DATA         = 'promotion_sent';

	/**
	 * @var WishlistFetcher
	 */
	protected $wishlist_fetcher;

	/**
	 * @var FilterAggregator
	 */
	private $filter_aggregator;

	/**
	 * @var string[]|null
	 */
	private $users = null;

	public function __construct(
		WishlistFetcher $wishlist_fetcher,
		FilterAggregator $filter_aggregator = null
	) {
		$this->wishlist_fetcher  = $wishlist_fetcher;
		$this->filter_aggregator = $filter_aggregator ?: new FilterAggregator();
	}

	/**
	 * {@inheritdoc}
	 */
	public function hooks() {
		add_action( 'add_meta_boxes', [ $this, 'register_meta_box' ] );
		add_action( 'save_post_' . PostTypeRegister::POST_TYPE_PROMOTION, [ $this, 'submit_email' ] );
	}

	/**
	 * @return void
	 * @internal
	 */
	public function register_meta_box() {
		add_meta_box(
			'flexible_wishlist_promotion_sender',
			__( 'Send promotion', 'flexible-wishlist-analytics' ),
			[ $this, 'print_metabox' ],
			PostTypeRegister::POST_TYPE_PROMOTION,
			'side',
			'default'
		);
	}

	/**
	 * @return void
	 * @internal
	 */
	public function print_metabox() {
		global $post;

		$users   = $this->get_users();
		$reports = get_post_meta( $post->ID, self::POST_META_PROMOTION_DATA, false );

		/* translators: %s: users count */
		$button_label = __( 'Send promotion (%s)', 'flexible-wishlist-analytics' );

		?>
		<?php if ( $reports ) : ?>
			<ul class="order_notes">
				<?php foreach ( array_reverse( $reports ) as $report ) : ?>
					<li class="note system-note">
						<div class="note_content">
							<p>
								<?php
								echo wp_kses_post(
								/* translators: %s: users count */
									sprintf( __( 'Sent to %s users.', 'flexible-wishlist-analytics' ), $report['users'] ?? 0 )
								);
								echo '<br>';
								echo esc_html( $report['note'] );
								?>
							</p>
						</div>
						<p class="meta">
							<abbr class="exact-date">
								<?php
								echo wp_kses_post(
									wp_date( get_option( 'date_format' ) . ' ' . get_option( 'time_format' ), $report['date'] )
								);
								?>
							</abbr>
						</p>
					</li>
				<?php endforeach; ?>
			</ul>
		<?php endif; ?>

		<input type="hidden"
			name="<?php echo esc_attr( self::INPUT_NAME_USERS ); ?>"
			value="<?php echo esc_attr( implode( ',', $users ) ); ?>">
		<p>
			<label for="<?php echo esc_attr( self::INPUT_NAME_NOTE ); ?>"
				style="display: block;">
				<?php echo esc_html( __( 'Add a description', 'flexible-wishlist-analytics' ) ); ?>
			</label>
		</p>
		<p>
			<textarea name="<?php echo esc_attr( self::INPUT_NAME_NOTE ); ?>"
				id="<?php echo esc_attr( self::INPUT_NAME_NOTE ); ?>"
				rows="3" style="width: 100%;"></textarea>
		</p>
		<button type="submit"
			class="button button-primary button-large"
			name="<?php echo esc_attr( self::SEND_PROMOTION_BUTTON_NAME ); ?>"
			value="<?php echo esc_attr( self::SEND_PROMOTION_BUTTON_VALUE ); ?>"
			<?php echo ( ! $users ) ? 'disabled' : ''; ?>
			data-fw-sender="<?php echo esc_attr( $button_label ); ?>"
			data-fw-sender-loading="<?php echo esc_attr( __( 'Please wait...', 'flexible-wishlist-analytics' ) ); ?>"
			data-fw-sender-api="<?php echo esc_attr( EmailCalculationEndpoint::get_route_url() ); ?>">
			<?php echo wp_kses_post( sprintf( $button_label, count( $users ) ) ); ?>
		</button>
		<button type="submit"
			class="button button-large"
			name="<?php echo esc_attr( self::SEND_PROMOTION_BUTTON_NAME ); ?>"
			value="<?php echo esc_attr( self::SEND_PROMOTION_BUTTON_VALUE_TEST ); ?>">
			<?php echo esc_html( __( 'Send test', 'flexible-wishlist-analytics' ) ); ?>
		</button>
		<?php
		if ( ( $_REQUEST[ self::SEND_PROMOTION_BUTTON_NAME ] ?? '' ) === self::SEND_PROMOTION_BUTTON_VALUE_TEST ) : // phpcs:ignore WordPress.Security
			?>
			<p>
				<strong><?php echo esc_html( __( 'Test e-mail sent!', 'flexible-wishlist-analytics' ) ); ?></strong>
			</p>
			<?php
		endif;
	}

	/**
	 * @return bool
	 * @internal
	 */
	public function submit_email(): bool {
		$email_mode = $_POST[ self::SEND_PROMOTION_BUTTON_NAME ] ?? ''; // phpcs:ignore WordPress.Security
		if ( $email_mode === '' ) {
			return false;
		}

		$is_test_mode     = ( $email_mode === self::SEND_PROMOTION_BUTTON_VALUE_TEST );
		$email_recipients = ( ! $is_test_mode ) ? $this->get_users() : [ get_current_user_id() ];

		if ( ! $email_recipients ) {
			return false;
		}

		global $post;
		$email_subject = get_post_meta( $post->ID, EmailMetaboxGenerator::POST_META_EMAIL_SUBJECT, true );
		$email_content = get_post_meta( $post->ID, EmailMetaboxGenerator::POST_META_EMAIL_CONTENT, true );

		if ( ! $email_subject || ! $email_content ) {
			return false;
		}

		\WC()->mailer();
		$status = ( new EmailSender( $email_recipients, $email_subject, $email_content ) )->trigger();

		if ( $status && ! $is_test_mode ) {
			add_post_meta(
				$post->ID,
				self::POST_META_PROMOTION_DATA,
				[
					'users' => count( $email_recipients ),
					'note'  => sanitize_text_field( $_REQUEST[ self::INPUT_NAME_NOTE ] ?? null ), // phpcs:ignore WordPress.Security
					'date'  => current_time( 'timestamp', true ), // phpcs:ignore WordPress.DateTime.CurrentTimeTimestamp.RequestedUTC
				],
				false
			);
		}

		return $status;
	}

	/**
	 * @return string[]
	 */
	private function get_users() {
		if ( $this->users !== null ) {
			return $this->users;
		}

		global $post;
		foreach ( $this->filter_aggregator->get_visible_filters() as $filter ) {
			$params[ $filter->get_name() ] = get_post_meta( $post->ID, $filter->get_name(), true );
		}
		$wishlists = $this->wishlist_fetcher->get_valid_wishlists( $params );

		$users = [];
		foreach ( $wishlists as $wishlist ) {
			if ( ! $wishlist->get_items() ) {
				continue;
			}

			$user = get_user_by( 'id', $wishlist->get_wp_user_id() );
			if ( ! $user ) {
				continue;
			}

			$users[] = $user->ID;
		}
		$this->users = array_unique( $users );

		return $this->users;
	}
}
