<?php

namespace WPDesk\FlexibleWishlistAnalytics\Table;

use WPDesk\FlexibleWishlistAnalytics\Filter\ProductNameFilter;
use WPDesk\FlexibleWishlistAnalytics\Filter\TargetTypeFilter;

/**
 * {@inheritdoc}
 */
class PopularProductsTable extends PopularTable {

	const PAGE_SLUG = 'flexible-wishlist-products';

	/**
	 * {@inheritdoc}
	 */
	public function get_slug(): string {
		return self::PAGE_SLUG;
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_hidden_fields(): array {
		return [
			TargetTypeFilter::FILTER_NAME => TargetTypeFilter::VALUE_PRODUCTS,
		];
	}

	/**
	 * {@inheritdoc}
	 */
	public function show_in_menu(): bool {
		return ( ! in_array( ( $_GET['page'] ?? '' ), [ PopularCategoriesTable::PAGE_SLUG, PopularTagsTable::PAGE_SLUG ], false ) ); // phpcs:ignore WordPress.Security
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_label(): string {
		return __( 'Popular products', 'flexible-wishlist-analytics' );
	}

	/**
	 * {@inheritdoc}
	 */
	public function is_menu_added(): bool {
		return true;
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_search_placeholder(): string {
		return __( 'Search product', 'flexible-wishlist-analytics' );
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_column_label( string $column_name ) {
		switch ( $column_name ) {
			case self::COLUMN_TITLE:
				return __( 'Product name', 'flexible-wishlist-analytics' );
		}

		return parent::get_column_label( $column_name );
	}

	/**
	 * {@inheritdoc}
	 */
	public function print_column_value( string $column_name, array $item_data ) {
		switch ( $column_name ) {
			case self::COLUMN_CHECKBOX:
				?>
				<input type="checkbox" name="post[]" value="<?php echo esc_attr( $item_data['item_id'] ); ?>">
				<?php
				break;
			case self::COLUMN_IMAGE:
				?>
				<a href="<?php echo esc_url( $item_data['product_url'] ); ?>" target="_blank">
					<?php echo wp_kses_post( $item_data['product_image'] ); ?>
				</a>
				<?php
				break;
			case self::COLUMN_TITLE:
				?>
				<a href="<?php echo esc_url( $item_data['product_url'] ); ?>" target="_blank">
					<?php echo esc_html( $item_data['title'] ); ?>
				</a>
				<?php if ( $item_data['product_stock_quantity'] !== null ) : ?>
				<p>
					<?php
					echo esc_html(
						sprintf(
						/* translators: %s: quantity value */
							__( 'Stock quantity: %s', 'flexible-wishlist-analytics' ),
							$item_data['product_stock_quantity']
						)
					);
					?>
				</p>
			<?php else : ?>
				<p>
					<?php
					echo esc_html(
						sprintf(
							'%1$s: %2$s',
							__( 'Stock status', 'flexible-wishlist-analytics' ),
							( $item_data['product_stock_status'] )
								? __( 'In stock', 'flexible-wishlist-analytics' )
								: __( 'Out of stock', 'flexible-wishlist-analytics' )
						)
					);
					?>
				</p>
			<?php endif; ?>
				<?php
				break;
		}

		return parent::print_column_value( $column_name, $item_data );
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_items(): array {
		$params = [
			TargetTypeFilter::FILTER_NAME  => TargetTypeFilter::VALUE_PRODUCTS,
			ProductNameFilter::FILTER_NAME => $_REQUEST['s'] ?? '', // phpcs:ignore WordPress.Security
		];
		foreach ( $this->get_filters() as $filter ) {
			$params[ $filter->get_name() ] = $_REQUEST[ $filter->get_name() ] ?? ''; // phpcs:ignore WordPress.Security
		}
		$wishlists = $this->wishlist_fetcher->get_valid_wishlists( $params );

		$items         = [];
		$product_users = [];
		foreach ( $wishlists as $wishlist ) {
			foreach ( $wishlist->get_items() as $wishlist_item ) {
				$product = wc_get_product( $wishlist_item->get_product_id() );
				if ( ! $product ) {
					continue;
				}

				if ( ! isset( $product_users[ $product->get_id() ] ) ) {
					$product_users[ $product->get_id() ] = [];
				}
				if ( ! in_array( $wishlist->get_wp_user_id(), $product_users[ $product->get_id() ], false ) ) {
					$product_users[ $product->get_id() ][] = $wishlist->get_wp_user_id();
				}

				$items[ $product->get_id() ] = [
					'item_id'                => $product->get_id(),
					'product_url'            => $product->get_permalink(),
					'title'                  => $product->get_name(),
					'product_image'          => $product->get_image( [ 40, 40 ] ),
					'product_stock_quantity' => $product->get_stock_quantity(),
					'product_stock_status'   => $product->is_in_stock(),
					'count_wishlists'        => ( $items[ $product->get_id() ]['count_wishlists'] ?? 0 ) + 1,
					'count_users'            => count( $product_users[ $product->get_id() ] ),
					'count_quantity'         => ( $items[ $product->get_id() ]['count_quantity'] ?? 0 ) + $wishlist_item->get_quantity(),
				];
			}
		}

		return $items;
	}
}
