<?php

namespace WPDesk\FlexibleWishlistAnalytics\Table;

use WPDesk\FlexibleWishlistAnalytics\Filter\ProductAddedAfterFilter;
use WPDesk\FlexibleWishlistAnalytics\Filter\ProductAddedBeforeFilter;
use WPDesk\FlexibleWishlistAnalytics\Filter\StockStatusFilter;
use WPDesk\FlexibleWishlistAnalytics\Service\WishlistFetcher;

/**
 * {@inheritdoc}
 */
abstract class PopularTable implements Table, HasDocumentationLink {

	const COLUMN_CHECKBOX        = 'cb';
	const COLUMN_IMAGE           = 'thumb';
	const COLUMN_TITLE           = 'title';
	const COLUMN_COUNT_WISHLISTS = 'count_wishlists';
	const COLUMN_COUNT_USERS     = 'count_users';
	const COLUMN_COUNT_QUANTITY  = 'count_quantity';

	/**
	 * @var WishlistFetcher
	 */
	protected $wishlist_fetcher;

	public function __construct( WishlistFetcher $wishlist_fetcher ) {
		$this->wishlist_fetcher = $wishlist_fetcher;
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_menu_label(): string {
		return __( 'Most popular', 'flexible-wishlist-analytics' );
	}

	public function get_documentation_link(): string {
		return __( 'https://wpde.sk/fw-popular-products-page-docs', 'flexible-wishlist-analytics' );
	}

	/**
	 * {@inheritdoc}
	 * @phpcs:disable WordPress.Security.ValidatedSanitizedInput
	 * @phpcs:disable WordPress.Security.NonceVerification
	 */
	public function get_menu_views(): array {
		return [
			sprintf(
				'%1$s' . __( 'Popular products', 'flexible-wishlist-analytics' ) . '%2$s',
				( ( $_GET['page'] ?? '' ) === PopularProductsTable::PAGE_SLUG )
					? ''
					: '<a href="' . admin_url( 'admin.php?page=' . PopularProductsTable::PAGE_SLUG ) . '">',
				( ( $_GET['page'] ?? '' ) === PopularProductsTable::PAGE_SLUG )
					? ''
					: '</a>'
			),
			sprintf(
				'%1$s' . __( 'Popular categories', 'flexible-wishlist-analytics' ) . '%2$s',
				( ( $_GET['page'] ?? '' ) === PopularCategoriesTable::PAGE_SLUG )
					? ''
					: '<a href="' . admin_url( 'admin.php?page=' . PopularCategoriesTable::PAGE_SLUG ) . '">',
				( ( $_GET['page'] ?? '' ) === PopularCategoriesTable::PAGE_SLUG )
					? ''
					: '</a>'
			),
			sprintf(
				'%1$s' . __( 'Popular tags', 'flexible-wishlist-analytics' ) . '%2$s',
				( ( $_GET['page'] ?? '' ) === PopularTagsTable::PAGE_SLUG )
					? ''
					: '<a href="' . admin_url( 'admin.php?page=' . PopularTagsTable::PAGE_SLUG ) . '">',
				( ( $_GET['page'] ?? '' ) === PopularTagsTable::PAGE_SLUG )
					? ''
					: '</a>'
			),
		];
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_columns(): array {
		return [
			self::COLUMN_CHECKBOX,
			self::COLUMN_IMAGE,
			self::COLUMN_TITLE,
			self::COLUMN_COUNT_WISHLISTS,
			self::COLUMN_COUNT_USERS,
			self::COLUMN_COUNT_QUANTITY,
		];
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_sortable_columns(): array {
		return [
			self::COLUMN_TITLE,
			self::COLUMN_COUNT_WISHLISTS,
			self::COLUMN_COUNT_USERS,
			self::COLUMN_COUNT_QUANTITY,
		];
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_column_label( string $column_name ) {
		switch ( $column_name ) {
			case self::COLUMN_CHECKBOX:
				return '<label class="screen-reader-text" for="cb-select-all-1">' . __( 'Select All', 'flexible-wishlist-analytics' ) . '</label><input id="cb-select-all-1" type="checkbox" />';
			case self::COLUMN_TITLE:
				return __( 'Title', 'flexible-wishlist-analytics' );
			case self::COLUMN_COUNT_WISHLISTS:
				return __( 'Wishlists', 'flexible-wishlist-analytics' );
			case self::COLUMN_COUNT_USERS:
				return __( 'Users', 'flexible-wishlist-analytics' );
			case self::COLUMN_COUNT_QUANTITY:
				return __( 'Quantity in wishlists', 'flexible-wishlist-analytics' );
		}

		return null;
	}

	/**
	 * {@inheritdoc}
	 */
	public function print_column_value( string $column_name, array $item_data ) {
		switch ( $column_name ) {
			case self::COLUMN_COUNT_WISHLISTS:
				echo esc_html( $item_data['count_wishlists'] );
				break;
			case self::COLUMN_COUNT_USERS:
				echo esc_html( $item_data['count_users'] );
				break;
			case self::COLUMN_COUNT_QUANTITY:
				echo esc_html( $item_data['count_quantity'] );
				break;
		}

		return null;
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_filters(): array {
		return [
			new StockStatusFilter(),
			new ProductAddedAfterFilter(),
			new ProductAddedBeforeFilter(),
		];
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_bulk_actions(): array {
		return [];
	}

	/**
	 * {@inheritdoc}
	 */
	public function init_bulk_actions( string $action ) {}
}
