<?php

namespace WPDesk\FlexibleWishlistAnalytics\Table;

use FlexibleWishlistAnalyticsVendor\WPDesk\PluginBuilder\Plugin\Hookable;
use WPDesk\FlexibleWishlistAnalytics\Exception\TemplateLoadingFailed;
use WPDesk\FlexibleWishlistAnalytics\Service\TemplateLoader;

class TableLoader implements Hookable {

	const MENU_PAGE_SLUG               = 'flexible-wishlist-settings';
	const SCREEN_OPTION_PER_PAGE_NAME  = 'flexible_wishlist_per_page';
	const SCREEN_OPTION_PER_PAGE_VALUE = 20;

	/**
	 * @var TemplateLoader
	 */
	private $template_loader;

	/**
	 * @var Table
	 */
	private $table;

	public function __construct( TemplateLoader $template_loader, Table $table ) {
		$this->template_loader = $template_loader;
		$this->table           = $table;
	}

	/**
	 * {@inheritdoc}
	 */
	public function hooks(): void {
		add_action( 'admin_menu', [ $this, 'create_admin_page' ] );
		add_action( 'load-flexible-wishlist_page_' . $this->table->get_slug(), [ $this, 'set_screen_options' ] );
		add_action( 'load-flexible-wishlist_page_' . $this->table->get_slug(), [ $this, 'init_bulk_actions' ] );
		add_filter( 'set_screen_option_' . self::SCREEN_OPTION_PER_PAGE_NAME, [ $this, 'update_screen_option' ], 10, 3 );
		add_filter( 'views_flexible-wishlist_page_' . $this->table->get_slug(), [ $this, 'update_sub_menu' ] );
	}

	/**
	 * @internal
	 */
	public function create_admin_page(): void {
		add_submenu_page(
			( $this->table->show_in_menu() ) ? self::MENU_PAGE_SLUG : '',
			$this->table->get_menu_label(),
			$this->table->get_menu_label(),
			'manage_options',
			$this->table->get_slug(),
			[ $this, 'print_admin_page' ]
		);
	}

	/**
	 * @throws TemplateLoadingFailed
	 */
	public function print_admin_page(): void {
		$this->template_loader->load_template(
			'admin-popular',
			[
				'wp_list_table' => new TableIntegrator( $this->table ),
				'page_title'    => $this->table->get_label(),
				'search_label'  => $this->table->get_search_placeholder(),
				'hidden_fields' => $this->table->get_hidden_fields(),
			]
		);
	}

	/**
	 * @return void
	 * @internal
	 */
	public function set_screen_options() {
		add_screen_option(
			'per_page',
			[
				'default' => self::SCREEN_OPTION_PER_PAGE_VALUE,
				'option'  => self::SCREEN_OPTION_PER_PAGE_NAME,
			]
		);
	}

	/**
	 * @return void
	 * @internal
	 */
	public function init_bulk_actions() {
		$action = $_REQUEST['action'] ?? ''; // phpcs:ignore WordPress.Security
		if ( $action === '' ) {
			return;
		}

		$this->table->init_bulk_actions( $action );
	}

	/**
	 * @return void
	 * @internal
	 */
	public function update_screen_option( $screen_option, $option_name, $option_value ) {
		switch ( $option_name ) {
			case self::SCREEN_OPTION_PER_PAGE_NAME:
				return $option_value;
			default:
				return $screen_option;
		}
	}

	/**
	 * @param string[] $views .
	 *
	 * @return string[]
	 * @internal
	 */
	public function update_sub_menu( array $views ): array {
		return $this->table->get_menu_views();
	}
}
