<?php

namespace WPDesk\FlexibleWishlistAnalytics\Table;

use WPDesk\FlexibleWishlist\Repository\SettingsRepository;
use WPDesk\FlexibleWishlist\Repository\WishlistRepository;
use WPDesk\FlexibleWishlist\Settings\Option\TextArchiveUrlOption;
use WPDesk\FlexibleWishlistAnalytics\Filter\TargetTypeFilter;
use WPDesk\FlexibleWishlistAnalytics\Service\EmailSender;
use WPDesk\FlexibleWishlistAnalytics\Service\WishlistFetcher;

/**
 * {@inheritdoc}
 */
class WishlistsArchiveTable implements Table, HasDocumentationLink {

	const PAGE_SLUG             = 'flexible-wishlist-all';
	const COLUMN_CHECKBOX       = 'cb';
	const COLUMN_TITLE          = 'title';
	const COLUMN_USER_EMAIL     = 'email';
	const COLUMN_COUNT_QUANTITY = 'count_quantity';
	const COLUMN_DATE_ADDED     = 'date_added';
	const COLUMN_DATE_EMAIL     = 'date_email';
	const COLUMN_ACTIONS        = 'actions';

	/**
	 * @var WishlistFetcher
	 */
	private $wishlist_fetcher;

	/**
	 * @var WishlistRepository
	 */
	private $wishlist_repository;

	/**
	 * @var mixed
	 */
	private $settings_repository;

	public function __construct(
		WishlistFetcher $wishlist_fetcher,
		WishlistRepository $wishlist_repository,
		SettingsRepository $settings_repository
	) {
		$this->wishlist_fetcher    = $wishlist_fetcher;
		$this->wishlist_repository = $wishlist_repository;
		$this->settings_repository = $settings_repository;
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_slug(): string {
		return self::PAGE_SLUG;
	}

	public function get_documentation_link(): string {
		return __( 'https://wpde.sk/fw-all-wishlists-page-docs', 'flexible-wishlist-analytics' );
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_hidden_fields(): array {
		return [
			TargetTypeFilter::FILTER_NAME => TargetTypeFilter::VALUE_WISHLIST_USER,
		];
	}

	/**
	 * {@inheritdoc}
	 */
	public function show_in_menu(): bool {
		return true;
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_label(): string {
		return __( 'All wishlists', 'flexible-wishlist-analytics' );
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_menu_label(): string {
		return $this->get_label();
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_menu_views(): array {
		return [];
	}

	/**
	 * {@inheritdoc}
	 */
	public function is_menu_added(): bool {
		return true;
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_columns(): array {
		return [
			self::COLUMN_CHECKBOX,
			self::COLUMN_TITLE,
			self::COLUMN_USER_EMAIL,
			self::COLUMN_COUNT_QUANTITY,
			self::COLUMN_DATE_ADDED,
			self::COLUMN_DATE_EMAIL,
			self::COLUMN_ACTIONS,
		];
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_sortable_columns(): array {
		return [
			self::COLUMN_TITLE,
			self::COLUMN_COUNT_QUANTITY,
			self::COLUMN_DATE_ADDED,
			self::COLUMN_DATE_EMAIL,
		];
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_column_label( string $column_name ) {
		switch ( $column_name ) {
			case self::COLUMN_CHECKBOX:
				return '<label class="screen-reader-text" for="cb-select-all-1">' . __( 'Select All', 'flexible-wishlist-analytics' ) . '</label><input id="cb-select-all-1" type="checkbox" />';
			case self::COLUMN_TITLE:
				return __( 'Title', 'flexible-wishlist-analytics' );
			case self::COLUMN_USER_EMAIL:
				return __( 'User e-mail', 'flexible-wishlist-analytics' );
			case self::COLUMN_COUNT_QUANTITY:
				return __( 'Product quantity', 'flexible-wishlist-analytics' );
			case self::COLUMN_DATE_ADDED:
				return __( 'Created date', 'flexible-wishlist-analytics' );
			case self::COLUMN_DATE_EMAIL:
				return __( 'Last promotion sent', 'flexible-wishlist-analytics' );
			case self::COLUMN_ACTIONS:
				return __( 'Actions', 'flexible-wishlist-analytics' );
		}

		return null;
	}

	/**
	 * {@inheritdoc}
	 */
	public function print_column_value( string $column_name, array $item_data ) {
		switch ( $column_name ) {
			case self::COLUMN_CHECKBOX:
				?>
				<input type="checkbox" name="post[]" value="<?php echo esc_attr( $item_data['item_id'] ); ?>">
				<?php
				break;
			case self::COLUMN_TITLE:
				?>
				<a href="<?php echo esc_html( $item_data['wishlist_url'] ); ?>">
					<?php echo esc_html( $item_data['title'] ); ?>
				</a>
				<?php
				break;
			case self::COLUMN_USER_EMAIL:
				?>
				<a href="<?php echo esc_url( $item_data['wishlist_url'] ); ?>" target="_blank">
					<?php echo esc_html( $item_data['user_email'] ?: '-' ); ?>
				</a>
				<?php
				break;
			case self::COLUMN_COUNT_QUANTITY:
				?>
				<?php echo esc_html( $item_data['count_quantity'] ); ?>
				<?php
				break;
			case self::COLUMN_DATE_ADDED:
				?>
				<?php echo esc_html( $item_data['date_added'] ); ?>
				<?php
				break;
			case self::COLUMN_DATE_EMAIL:
				?>
				<?php echo esc_html( $item_data['date_email'] ?: '-' ); ?>
				<?php
				break;
			case self::COLUMN_ACTIONS:
				?>
				<?php if ( $item_data['user_email'] ) : ?>
				<form>
					<input type="hidden" name="page"
						value="<?php echo esc_attr( $_GET['page'] ?? '' ); // phpcs:ignore WordPress.Security ?>">
					<input type="hidden" name="target_type"
						value="<?php echo esc_attr( TargetTypeFilter::VALUE_WISHLIST_USER ); ?>">
					<input type="hidden" name="post_target[]"
						value="<?php echo esc_attr( $item_data['user_id'] ); ?>">
					<button type="submit" class="button" name="fw_send_promotion" value="1">
						<?php echo esc_html__( 'Create Promotion', 'flexible-wishlist-analytics' ); ?>
					</button>
				</form>
			<?php endif; ?>
				<?php
				break;
		}

		return null;
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_filters(): array {
		return [];
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_search_placeholder() {
		return null;
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_items(): array {
		$wishlists = $this->wishlist_fetcher->get_valid_wishlists( [] );
		$items     = [];
		foreach ( $wishlists as $wishlist ) {
			$user       = get_user_by( 'id', $wishlist->get_wp_user_id() );
			$email_date = ( $user ) ? get_user_meta( $user->ID, EmailSender::USER_META_LATEST_PROMOTION, true ) : null;

			$items[] = [
				'item_id'        => $wishlist->get_id(),
				'title'          => $wishlist->get_name(),
				'wishlist_url'   => get_home_url( null, $this->settings_repository->get_value( TextArchiveUrlOption::FIELD_NAME ) . '/' . $wishlist->get_list_token() ),
				'user_id'        => ( $user ) ? $user->ID : null,
				'user_email'     => ( $user ) ? $user->get( 'user_email' ) : null,
				'count_quantity' => count( $wishlist->get_items() ),
				'date_added'     => $wishlist->get_created_at()->format( 'Y-m-d' ),
				'date_email'     => ( $email_date ) ? $email_date->format( 'Y-m-d H:i:s' ) : null,
			];
		}

		return $items;
	}

	/**
	 * {@inheritdoc}
	 */
	public function get_bulk_actions(): array {
		return [
			'delete' => __( 'Delete', 'flexible-wishlist-analytics' ),
		];
	}

	/**
	 * {@inheritdoc}
	 */
	public function init_bulk_actions( string $action ) {
		if ( $action === 'delete' ) {
			$post_ids = array_filter( $_REQUEST['post'] ?? [] ); // phpcs:ignore WordPress.Security
			foreach ( $post_ids as $post_id ) {
				$wishlist = $this->wishlist_repository->get_by_id( $post_id );
				if ( $wishlist === null ) {
					continue;
				}
				$this->wishlist_repository->remove( $wishlist );
			}
		}
	}
}
